unit Control2D;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, INIFiles, fparse;

type
  TControlForm = class(TForm)
    GBFunctions: TGroupBox;
    FunctionsBox: TListBox;
    GBEditFunction: TGroupBox;
    CBMulti: TCheckBox;
    EMultiEnd: TEdit;
    EMultiStep: TEdit;
    PColor1: TPanel;
    BChangeFunction: TBitBtn;
    BAddFunction: TBitBtn;
    BShowFunction: TBitBtn;
    BDelFunction: TBitBtn;
    LMultiTo: TLabel;
    LMultiStep: TLabel;
    PColor2: TPanel;
    CBColor2: TCheckBox;
    GBViewRange: TGroupBox;
    EXRangeStart: TEdit;
    EXRangeEnd: TEdit;
    LXRange: TLabel;
    LXRangeTo: TLabel;
    EYRangeEnd: TEdit;
    LYRangeTo: TLabel;
    EYRangeStart: TEdit;
    LYRange: TLabel;
    EMultiStart: TEdit;
    ColorSelect: TColorDialog;
    BApplyRange: TBitBtn;
    BResetRange: TBitBtn;
    PGrid: TPanel;
    LXRangeStep: TLabel;
    LYRangeStep: TLabel;
    EXRangeStep: TEdit;
    EYRangeStep: TEdit;
    Bevel1: TBevel;
    CBAutoGrid: TCheckBox;
    PAxeColor: TPanel;
    CBAxes: TCheckBox;
    CBGrid: TCheckBox;
    PGridColor: TPanel;
    CBScale: TCheckBox;
    Bevel2: TBevel;
    CBForegroundScale: TCheckBox;
    CBDerive: TCheckBox;
    LColor: TLabel;
    EFunction: TEdit;
    GBImpex: TGroupBox;
    BImportG2D: TBitBtn;
    BExportG2D: TBitBtn;
    BExportPS: TBitBtn;
    BExportBMP: TBitBtn;
    DSaveG2D: TSaveDialog;
    DOpenG2D: TOpenDialog;
    DSaveBMP: TSaveDialog;
    PParamSwitch: TPanel;
    PParametric: TPanel;
    EParamX: TEdit;
    EParamY: TEdit;
    LParametric: TLabel;
    Bevel3: TBevel;
    LTRange: TLabel;
    ETRangeStart: TEdit;
    LTRangeTo: TLabel;
    LTRangeStep: TLabel;
    ETRangeStep: TEdit;
    BApplyTRange: TBitBtn;
    BResetTRange: TBitBtn;
    ETRangeEnd: TEdit;
    procedure FunctionsBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure BAddFunctionClick(Sender: TObject);
    procedure CBMultiClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure CBColor2Click(Sender: TObject);
    procedure PColorClick(Sender: TObject);
    procedure BChangeFunctionClick(Sender: TObject);
    procedure FFunctionKeyPress(Sender: TObject; var Key: Char);
    procedure BShowFunctionClick(Sender: TObject);
    procedure FunctionsBoxDblClick(Sender: TObject);
    procedure BDelFunctionClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure BResetRangeClick(Sender: TObject);
    procedure CBAutoGridClick(Sender: TObject);
    procedure BApplyRangeClick(Sender: TObject);
    procedure ERangeKeyPress(Sender: TObject; var Key: Char);
    procedure SetupChange(Sender: TObject);
    procedure FunctionsBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure BImportG2DClick(Sender: TObject);
    procedure BExportG2DClick(Sender: TObject);
    procedure BExportBMPClick(Sender: TObject);
    procedure BExportPSClick(Sender: TObject);
    procedure PParamSwitchMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PParamSwitchMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PParamSwitchClick(Sender: TObject);
    procedure BResetTRangeClick(Sender: TObject);
    procedure BApplyTRangeClick(Sender: TObject);
    procedure ETRangeKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    fnChanged:boolean;
    LCB_Unchecked,LCB_Checked:TBitmap;
  public
    { Public declarations }
  end;

var
  ControlForm: TControlForm;

implementation
uses Manage2D, GraphW2D,PostScript2D, PSUI2D;

{$R check.res}
{$R *.dfm}

{$I ImportG2D.inc.pas}
{$I ExportG2D.inc.pas}

procedure TControlForm.FunctionsBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var s:string;
begin
  Index:=StrToInt(FunctionsBox.Items[Index]);
  with FunctionsBox,Canvas,Functions[Index],Rect do begin
    FillRect(Rect);
    if Parametric then begin
      if Derive then s:='x(t)=d['+Expression+']/dt; y(t)=d['+Expression2+']/dt'
                else s:='x(t)='+Expression+'; y(t)='+Expression2;
    end else begin
      if Derive then s:='f(x)=d['+Expression+']/dx'
                else s:='f(x)='+Expression;
    end;
    if Multi then s:=s+'; a='+FloatToStr(MultiStart)+'..'+FloatToStr(MultiEnd)+
                       ' ('+FloatToStr(MultiStep)+')';
    TextOut(Left+58,Top+1,s);
    Pen.Style:=psSolid;
    Pen.Color:=0;
    Pen.Width:=1;
    Brush.Style:=bsSolid;
    Brush.Color:=Color;
    Rectangle(Left+26,Top+3,Left+47,Top+14);
    if Gradient then begin
      Brush.Color:=Color2;
      FillRect(Classes.Rect(Left+36,Top+4,Left+46,Top+13));
    end;
    if Error OR (Index=ActiveFunction) then begin
      Brush.Style:=bsClear;
      Font.Style:=[fsBold];
    end;
    if Error then TextOut(Left+50,Top+1,'!!');
    if (Index=ActiveFunction) then TextOut(Left+16,Top+1,#187);
    if Active then Draw(Left+2,Top+2,LCB_Checked)
              else Draw(Left+2,Top+2,LCB_Unchecked);
  end;
end;

procedure TControlForm.BAddFunctionClick(Sender: TObject);
var Index:integer;
begin
  Index:=CreateNewFunction;
  FormToFunction(Index);
  FunctionsBox.ItemIndex:=FunctionsBox.Items.Add(IntToStr(Index));
  ActiveFunction:=Index;
  FunctionsBox.Invalidate;
  GraphForm.Invalidate;
  fnChanged:=false;
end;

procedure TControlForm.CBMultiClick(Sender: TObject);
var B:boolean;
begin
  B:=CBMulti.Checked;
  EMultiStart.Enabled:=B;
  LMultiTo.Enabled:=B;
  EMultiEnd.Enabled:=B;
  LMultiStep.Enabled:=B;
  EMultiStep.Enabled:=B;
  if Sender<>nil then fnChanged:=true;
end;

procedure TControlForm.FormShow(Sender: TObject);
begin
  CBMultiClick(nil);
  CBColor2Click(nil);
  CBAutoGridClick(nil);
  SetupChange(Sender);
  BResetTRangeClick(nil);
  fnChanged:=false;
  GraphForm.Show;
end;

procedure TControlForm.CBColor2Click(Sender: TObject);
begin
  PColor2.Enabled:=CBColor2.Checked;
  if Sender<>nil then fnChanged:=true;
end;

procedure TControlForm.PColorClick(Sender: TObject);
begin
  with Sender as TPanel do begin
    ColorSelect.Color:=Color;
    if ColorSelect.Execute then begin
      Color:=ColorSelect.Color;
      case Tag of
        0:fnChanged:=true;
        1:SetupChange(Sender);
      end;
    end;
  end;
end;

procedure TControlForm.BChangeFunctionClick(Sender: TObject);
begin
  if ActiveFunction<0 then begin
    if Sender=nil then BAddFunctionClick(Sender);
    exit;
  end;
  FormToFunction(ActiveFunction);
  FunctionsBox.Invalidate;
  GraphForm.Invalidate;
  fnChanged:=false;
end;

procedure TControlForm.FFunctionKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then begin
    BChangeFunctionClick(nil);
    Key:=#0;
  end;
  if Key IN [^H,^I,#32..#255] then fnChanged:=true;
end;

procedure TControlForm.BShowFunctionClick(Sender: TObject);
var res:integer;
begin
  if FunctionsBox.ItemIndex<0 then exit;
  if fnChanged then begin
    res:=MessageDlg(
           'Die Funktion wurde gendert! Vorher bernehmen?',
           mtWarning,[mbYes,mbNo,mbCancel],0);
    if res=mrCancel then exit;
    if res=mrYes then BChangeFunctionClick(Sender);
  end;
  ActiveFunction:=StrToInt(FunctionsBox.Items[FunctionsBox.ItemIndex]);
  FunctionToForm(ActiveFunction);
  FunctionsBox.Invalidate;
  fnChanged:=false;
end;

procedure TControlForm.FunctionsBoxDblClick(Sender: TObject);
begin
  BShowFunctionClick(Sender);
end;

procedure TControlForm.BDelFunctionClick(Sender: TObject);
var i:integer;
begin
  if ActiveFunction<0 then exit;
  i:=FunctionsBox.Items.IndexOf(IntToStr(ActiveFunction));
  if i<0 then exit;
  FunctionsBox.Items.Delete(i);
  Functions[ActiveFunction].Used:=false;
  ActiveFunction:=-1;
  FunctionsBox.Invalidate;
  GraphForm.Invalidate;
end;

procedure TControlForm.FormCreate(Sender: TObject);
begin
//AllocConsole;
  DecimalSeparator:='.';
  LCB_Unchecked:=TBitmap.Create;
  LCB_Unchecked.LoadFromResourceName(HInstance,'LCB_UNCHECKED');
  LCB_Unchecked.Transparent:=true;
  LCB_Checked:=TBitmap.Create;
  LCB_Checked.LoadFromResourceName(HInstance,'LCB_CHECKED');
  LCB_Checked.Transparent:=true;
end;
                         
procedure TControlForm.FormDestroy(Sender: TObject);
begin
  LCB_Checked.Free;
  LCB_Unchecked.Free;
end;

procedure TControlForm.BResetRangeClick(Sender: TObject);
begin
  ResetViewRange;
  GraphForm.RangeChange;
end;

procedure TControlForm.CBAutoGridClick(Sender: TObject);
begin
  AutoGrid:=CBAutoGrid.Checked;
  LXRangeStep.Enabled:=not(AutoGrid);
  EXRangeStep.Enabled:=not(AutoGrid);
  LYRangeStep.Enabled:=not(AutoGrid);
  EYRangeStep.Enabled:=not(AutoGrid);
end;

procedure TControlForm.BApplyRangeClick(Sender: TObject);
var Xmin,Xmax,Xstep,Ymin,Ymax,Ystep:double;
begin
  EditToFloat(EXRangeStart,Xmin);
  EditToFloat(EXRangeEnd,Xmax);
  if not AutoGrid then EditToFloat(EXRangeStep,Xstep) else Xstep:=StepX;
  EditToFloat(EYRangeStart,Ymin);
  EditToFloat(EYRangeEnd,Ymax);
  if not AutoGrid then EditToFloat(EYRangeStep,Ystep) else Ystep:=StepY;
  GraphForm.SetViewRange(Xmin,Xmax,Xstep,Ymin,Ymax,Ystep);
end;

procedure TControlForm.ERangeKeyPress(Sender: TObject; var Key: Char);
begin
  if Key<>#13 then exit;
  Key:=#0;
  BApplyRangeClick(Sender);
end;

var SetupChangeLock:boolean=false;
procedure TControlForm.SetupChange(Sender: TObject);
begin
  if SetupChangeLock then exit;
  if Sender=nil then begin
    SetupChangeLock:=true;
    CBAxes.Checked:=DrawAxes;
    CBGrid.Checked:=DrawGrid;
    CBScale.Checked:=DrawScale;
    PAxeColor.Color:=AxeColor;
    PGridColor.Color:=GridColor;
    CBForegroundScale.Checked:=ForegroundScale;
    SetupChangeLock:=false;
  end else begin
    DrawAxes:=CBAxes.Checked;
    DrawGrid:=CBGrid.Checked;
    DrawScale:=CBScale.Checked;
    AxeColor:=PAxeColor.Color;
    GridColor:=PGridColor.Color;
    ForegroundScale:=CBForegroundScale.Checked;
    GraphForm.Invalidate;
  end;
  PAxeColor.Enabled:=DrawAxes;
  PGridColor.Enabled:=DrawGrid;
  CBScale.Enabled:=DrawAxes;
  CBForegroundScale.Enabled:=DrawAxes AND DrawScale;
end;

procedure TControlForm.FunctionsBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var Index:integer;
begin
  if X>17 then exit;
  Index:=FunctionsBox.ItemAtPos(Point(X,Y),true);
  if Index<0 then exit;
  Index:=StrToInt(FunctionsBox.Items[Index]);
  with Functions[Index] do Active:=not(Active);
  FunctionsBox.Invalidate;
  GraphForm.Invalidate;
end;

procedure TControlForm.BExportBMPClick(Sender: TObject);
var BMP:TBitmap;
begin
  if not DSaveBMP.Execute then exit;
  GraphForm.SetFocus;
  GraphForm.Repaint;
  BMP:=GraphForm.GetFormImage;
  try
    BMP.SaveToFile(DSaveBMP.FileName);
  except
    MessageDlg('Fehler beim Speichern der Bitmap-Datei!',mtError,[mbOK],0);
  end;
  BMP.Free;
  SetFocus;
end;

procedure TControlForm.BExportPSClick(Sender: TObject);
begin
  PSUIForm.Visible:=not(PSUIForm.Visible);
end;

procedure TControlForm.PParamSwitchMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  (Sender as TPanel).BevelInner:=bvLowered;
end;

procedure TControlForm.PParamSwitchMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  (Sender as TPanel).BevelInner:=bvRaised;
end;

procedure TControlForm.PParamSwitchClick(Sender: TObject);
begin
  if PParametric.Visible then begin
    PParamSwitch.Caption:='f(x)=';
    PParametric.Visible:=false;
    EFunction.Visible:=true;
  end else begin
    PParamSwitch.Caption:='x(t)=';
    PParametric.Visible:=true;
    EFunction.Visible:=false;
  end;
end;

procedure TControlForm.BResetTRangeClick(Sender: TObject);
begin
  ETRangeStart.Text:='0';
  ETRangeEnd.Text:='2*pi';
  ETRangeStep.Text:='pi/50';
  BApplyTRangeClick(Sender);
end;

procedure TControlForm.BApplyTRangeClick(Sender: TObject);
var Tmin,Tmax,Tstep:double;
begin
  EditToFloat(ETRangeStart,Tmin);
  EditToFloat(ETRangeEnd,Tmax);
  EditToFloat(ETRangeStep,Tstep);
  GraphForm.SetTRange(Tmin,Tmax,Tstep); 
end;

procedure TControlForm.ETRangeKeyPress(Sender: TObject; var Key: Char);
begin
  if Key<>#13 then exit;
  Key:=#0;
  BApplyTRangeClick(Sender);
end;

end.
