unit Control3D;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, fparse, Buttons;

const SCALESTEP=1.2589254117941672104239541063958;
      MAXPIXELS=4180000;

type TMoveMode=(mmNone,mmPan,mmScale);

type TMat2=array[1..2,1..2]of double;
function Mat2(a11,a12,a21,a22:double):TMat2;
function Invert(const m:TMat2):TMat2;

type
  TControlForm = class(TForm)
    PDrawZone: TPanel;
    DrawZone: TPaintBox;
    EFunction: TEdit;
    LZIs: TLabel;
    BResX2: TBitBtn;
    BResD2: TBitBtn;
    BRMin: TBitBtn;
    BRMax: TBitBtn;
    LCenter: TLabel;
    BReset: TBitBtn;
    LScale: TLabel;
    LMisc: TLabel;
    BExport: TBitBtn;
    DSaveBMP: TSaveDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DrawZonePaint(Sender: TObject);
    procedure EFunctionKeyPress(Sender: TObject; var Key: Char);
    procedure BResD2Click(Sender: TObject);
    procedure BResX2Click(Sender: TObject);
    procedure BRMinClick(Sender: TObject);
    procedure BRMaxClick(Sender: TObject);
    procedure DrawZoneMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DrawZoneMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure DrawZoneMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure BResetClick(Sender: TObject);
    procedure BExportClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    hCache:array of integer;
    AX,AY,BX,BY:integer;
    SX,SY:integer;
    WX,WY:integer;
    CX,CY,Pitch,Offset:integer;
    ScaleX,ScaleY,HScale:double;
    CenterX,CenterY:double;
    Alpha,Beta,Lambda:double;
    Tol:integer;
    Tree:PExpTreeNode;
    MoveMode:TMoveMode;
    MMBCX,MMBCY:double; MMBMX,MMBMY:integer;
    MMPM2:TMat2;
    DrawLock:boolean;
    function GetX(IX:integer):double;
    function GetY(IY:integer):double;
  public
    { Public declarations }
    procedure FunctionChange;
    procedure RangeChange;
    procedure DrawIt(Canvas:TCanvas);
  end;

var
  ControlForm: TControlForm;

implementation

{$R *.dfm}

function Mat2(a11,a12,a21,a22:double):TMat2;
begin
  Result[1,1]:=a11; Result[1,2]:=a12;
  Result[2,1]:=a21; Result[2,2]:=a22;
end;

function Invert(const m:TMat2):TMat2;
var det:double;
begin
  det:=m[1,1]*m[2,2]-m[1,2]*m[2,1];
  Result[1,1]:=m[2,2]/det;
  Result[1,2]:=-m[1,2]/det;
  Result[2,1]:=-m[2,1]/det;
  Result[2,2]:=m[1,1]/det;
end;

procedure TControlForm.FormCreate(Sender: TObject);
begin
//AllocConsole;
  DecimalSeparator:='.';
  Tree:=nil;
  MoveMode:=mmNone;
  DrawLock:=true;
  FunctionChange;
  BResetClick(nil);
  DrawLock:=false;
end;

procedure TControlForm.FormDestroy(Sender: TObject);
begin
//
end;

function TControlForm.GetX(IX:integer):double; begin GetX:=CenterX+ScaleX*IX; end;
function TControlForm.GetY(IY:integer):double; begin GetY:=CenterY+ScaleY*IY; end;

procedure TControlForm.DrawIt(Canvas:TCanvas);
var A,B,D,MX,MY,MinH,MaxH,dH,h:integer;
    h1,h2,h3,h4:integer; X,Y:double;
  function idx(A,B:integer):integer; begin Result:=Offset+Pitch*B+A; end;
const INVALID=maxint;
begin with Canvas do begin
  MinH:=maxint; MaxH:=-maxint;
  for B:=-SY to SY do for A:=-SX to SX do begin
    X:=GetX(A); Y:=GetY(B);
    Variables['x']:=X;
    Variables['y']:=Y;
    Variables['r']:=sqrt(sqr(X)+sqr(Y));
    try
      h:=round(HScale*Evaluate(Tree));
      if MathError OR (h<-Tol) OR (h>Tol) then h:=INVALID;
    except
      h:=INVALID;
    end;
    hCache[idx(A,B)]:=h;
    if h<>INVALID then begin
      if h<MinH then MinH:=h;
      if h>MaxH then MaxH:=h;
    end;
  end;
  MinH:=MinH*4; MaxH:=MaxH*4;
  dH:=MaxH-MinH; if dH=0 then dH:=1;
  if MoveMode<>mmNone then begin
    Pen.Style:=psSolid;
    Pen.Color:=$FFFFFF;
  end else begin
    Pen.Style:=psSolid;
    Pen.Color:=$100000;
  end;
  for D:=-SX-SY to SX+SY do
    for A:=-SX to (SX-1) do begin
      B:=A-D;
      if (B>=-SY) AND (B<SY) then begin
        MX:=CX+AX*A+BX*B; MY:=CY+AY*A+BY*B;
        if (MX<WX) AND (MX+AX+BX>0) then begin
          h1:=hCache[idx(A,B)];
          h2:=hCache[idx(A+1,B)];
          h3:=hCache[idx(A+1,B+1)];
          h4:=hCache[idx(A,B+1)];
          if (h1<>INVALID) AND (h2<>INVALID) AND (h3<>INVALID) AND (h4<>INVALID) then begin
            if MoveMode=mmNone then begin
              Brush.Color:=$FF+((h1+h2+h3+h4-MinH)*255 DIV dH)SHL 8;
              Polygon([
                Point(MX,MY-h1),
                Point(MX+AX,MY+AY-h2),
                Point(MX+AX+BX,MY+AY+BY-h3),
                Point(MX+BX,MY+BY-h4)
              ]);
            end else begin
              MoveTo(MX+AX,MY+AY-h2);
              LineTo(MX,MY-h1);
              LineTo(MX+BX,MY+BY-h4);
            end;
          end;
        end;
      end;
    end;
end; end;

procedure TControlForm.DrawZonePaint(Sender: TObject);
begin
  if DrawLock then exit;
  DrawIt(DrawZone.Canvas);
end;

procedure TControlForm.EFunctionKeyPress(Sender: TObject; var Key: Char);
begin
  if Key<>#13 then exit;
  Key:=#0; FunctionChange;
end;

procedure TControlForm.FunctionChange;
begin
  DestroyExpressionTree(Tree);
  Tree:=ParseExpression(EFunction.Text);
  if ParseError then begin
    DestroyExpressionTree(Tree);
    EFunction.Color:=clRed;
  end else begin
    EFunction.Color:=clWindow;
  end;
  DrawZone.Invalidate;
end;

procedure TControlForm.BResD2Click(Sender: TObject);
begin
  if (SX<2) OR (SY<2) then exit;
  SX:=SX DIV 2; SY:=SY DIV 2;
  ScaleX:=ScaleX*2.0; ScaleY:=ScaleY*2.0;
  Lambda:=Lambda*2.0;
  RangeChange;
end;

procedure TControlForm.BResX2Click(Sender: TObject);
begin
  if Lambda<2.0 then exit;
  SX:=SX*2; SY:=SY*2;
  ScaleX:=ScaleX*0.5; ScaleY:=ScaleY*0.5;
  Lambda:=Lambda*0.5;
  RangeChange;
end;

procedure TControlForm.BRMinClick(Sender: TObject);
begin
  if SX>0 then dec(SX);
  if SY>0 then dec(SY);
  RangeChange;
end;

procedure TControlForm.BRMaxClick(Sender: TObject);
begin
  inc(SX); inc(SY);
  RangeChange;
end;

procedure TControlForm.RangeChange;
begin
  WX:=DrawZone.Width; WY:=DrawZone.Height;
  CX:=WX DIV 2; CY:=WY DIV 2;
  Tol:=CY*3;
  AX:=round(cos(Alpha)*Lambda); AY:=round(sin(Alpha)*Lambda);
  BX:=round(cos(Beta)*Lambda); BY:=-round(sin(Beta)*Lambda);
  Pitch:=2*SX+1; Offset:=SY*Pitch+SX;
  SetLength(hCache,Pitch*(2*SY+1));
  DrawZone.Invalidate;
  LCenter.Caption:=
    'Zentrum X='+FloatToStr(CenterX)+' Y='+FloatToStr(CenterY);
  LScale.Caption:=
    'Skalierung X='+FloatToStr(ScaleX)+'/tick Y='+FloatToStr(ScaleY)+'/tick';
  LMisc.Caption:=
    'Hhenfaktor h='+FloatToStr(HScale)+' px/1  Feldgre Lambda='+
    FloatToStr(Lambda)+' px/tick';
end;

procedure TControlForm.DrawZoneMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MMBMX:=X; MMBMY:=Y;
  MMBMX:=X; MMBMY:=Y;
  if Button=mbLeft then begin
    MoveMode:=mmPan;
    MMBCX:=CenterX; MMBCY:=CenterY;
    MMPM2:=Invert(Mat2(AX,BX,AY,BY));
  end else if Button=mbRight then begin
    MoveMode:=mmScale;
    MMBCX:=Lambda; MMBCY:=HScale;
  end else begin
    MoveMode:=mmNone;
  end;
end;

procedure TControlForm.DrawZoneMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if MoveMode=mmNone then exit;
  MoveMode:=mmNone;
  DrawZone.Invalidate;
end;

procedure TControlForm.DrawZoneMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if MoveMode=mmNone then exit
  else if MoveMode=mmPan then begin
    CenterX:=MMBCX-ScaleX*(MMPM2[1,1]*(X-MMBMX)+MMPM2[1,2]*(Y-MMBMY));
    CenterY:=MMBCY-ScaleY*(MMPM2[2,1]*(X-MMBMX)+MMPM2[2,2]*(Y-MMBMY));
    RangeChange;
  end else if MoveMode=mmScale then begin
    HScale:=MMBCY+0.1*(MMBMY-Y);
    if HScale<0.0 then HScale:=0.0;
    Lambda:=MMBCX+(X-MMBMX) DIV 10;
    if Lambda<1.0 then Lambda:=1.0;
    RangeChange;
  end;
end;

procedure TControlForm.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if WheelDelta>0 then begin
    ScaleX:=ScaleX/SCALESTEP;
    ScaleY:=ScaleY/SCALESTEP;
  end else begin
    ScaleX:=ScaleX*SCALESTEP;
    ScaleY:=ScaleY*SCALESTEP;
  end;
  RangeChange;  
end;

procedure TControlForm.BResetClick(Sender: TObject);
begin
  SX:=20; SY:=20;
  ScaleX:=0.25; ScaleY:=0.25;
  CenterX:=0.0; CenterY:=0.0;
  Alpha:=pi/6.0; Beta:=Alpha; Lambda:=10.0;
  HScale:=20.0;
  RangeChange;
end;

procedure TControlForm.BExportClick(Sender: TObject);
var BMP:TBitmap; szf:string;
    Factor,OldLambda,OldHScale:double;
begin
  DrawLock:=true;
  if not DSaveBMP.Execute then begin DrawLock:=false; exit; end;
  szf:='1';
  if not InputQuery('Grenfaktor','Bitte geben Sie den Grenfaktor ein.'^M^M+
    'Sie knnen die Bitmap in einem Vielfachen der aktuellen Bildschirm'+
    'auflsung rendern lassen. Bitte geben Sie hier ein, um wieviel grer '+
    'die Ausgabe im Verhltnis zum aktuellen Bildschirmfenster sein soll.',szf) then
    begin
      DrawLock:=false;
      exit;
    end;
  try
    Factor:=StrToFloat(szf);
    Assert(Factor>=1.0);
  except
    MessageDlg('Der eingegebene Grenfaktor ist ungltig!',mtError,[mbOK],0);
    DrawLock:=false;
    exit;
  end;
  if (DrawZone.Width*DrawZone.Height*sqr(Factor)>MAXPIXELS)  then begin
    MessageDlg('Sorry, aber aufgrund einer besch******* Beschrnkung in '+
               'Windows'^M'knnen keine Bitmaps grer als ca. '+
               IntToStr(MAXPIXELS)+' Pixel erzeugt werden }:<',mtInformation,
               [mbOK],0);
    DrawLock:=false; exit;
  end;
  OldLambda:=Lambda; OldHScale:=HScale;
  Lambda:=Lambda*Factor; HScale:=HScale*Factor;
  WX:=round(Factor*DrawZone.Width);
  WY:=round(Factor*DrawZone.Height);
  CX:=WX DIV 2; CY:=WY DIV 2;
  Tol:=CY*3;
  AX:=round(cos(Alpha)*Lambda); AY:=round(sin(Alpha)*Lambda);
  BX:=round(cos(Beta)*Lambda); BY:=-round(sin(Beta)*Lambda);
  BMP:=TBitmap.Create;
  try with BMP do begin
    Width:=WX; Height:=WY;
    PixelFormat:=pfDevice;
    with Canvas do begin
      Brush.Color:=0;
      FillRect(Rect(0,0,WX,WY));
    end;
    DrawIt(Canvas);
    SaveToFile(DSaveBMP.FileName);
  end finally
    BMP.Free;
  end;
  Lambda:=OldLambda; HScale:=OldHScale;
  RangeChange;
  DrawLock:=false;
end;

procedure TControlForm.FormResize(Sender: TObject);
begin
  RangeChange;
end;

end.
