unit GraphW2D;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Math, fparse;

const SCALESTEP=1.2589254117941672104239541063958;
      AUTOGRIDSPEC=12.0;
      LN10=2.30258509299404568401799145468436;
      TICKSIZE=5;
      TEXTSPACINGY=1;
      TEXTSPACINGX=2;
      DIFFSTEPFACTOR=0.1;
      PARAMDIFFSTEPFACTOR=0.1;

type
  TGraphForm = class(TForm)
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    SX,SY:integer;  // Fenstergre (ClientWidth/-Height)
    CX,CY:integer;  // Fenstermitte (ClientWidth/-Height DIV 2)
    MinX,MinY,MaxX,MaxY:double;  // sichtbarer Ausschnitt
    GXmin,GXmax,GYmin,GYmax:integer;  // sichtbarer Gitterausschnitt
    RXmin,RXmax,RYmin,RYmax:double;  // maximale Funktionswerte (alles auerhalb = "unstetig")
    DiffStep,ParamDiffStep:double;  // Schrittweite zur numerischen Differentiation
    MouseMove:boolean;  // Flag: Mausverschiebung
    MDX,MDY:integer;  // Basis-Mauskoordinaten bei Verschiebung
    MBX,MBY:double;  // Basis-Mittelkoordinaten bei Verschiebung
    procedure DrawFunction(Tree:PExpTreeNode;Derive:boolean;GColor:TColor);
    procedure DrawParametric(TreeX,TreeY:PExpTreeNode;Derive:boolean;GColor:TColor);
  public
    { Public declarations }
    function GetX(X:double):integer;
    function GetY(Y:double):integer;
    function InvX(IX:integer):double;
    function InvY(IY:integer):double;
    procedure RangeChange;
    procedure SetViewRange(Xmin,Xmax,Xstep,Ymin,Ymax,Ystep:double);
    procedure SetTRange(Tmin,Tmax,Tstep:double);
  end;

var
  GraphForm: TGraphForm;

implementation
uses Manage2D, Control2D;

{$R *.dfm}

function TGraphForm.GetX(X:double):integer;
begin
  GetX:=CX+round((X-CenterX)*ScaleX);
end;

function TGraphForm.GetY(Y:double):integer;
begin
  GetY:=CY-round((Y-CenterY)*ScaleY);
end;

function TGraphForm.InvX(IX:integer):double;
begin
  InvX:=(IX-CX)/ScaleX+CenterX;
end;

function TGraphForm.InvY(IY:integer):double;
begin
  InvY:=(CY-IY)/ScaleY+CenterY;
end;

procedure TGraphForm.DrawFunction(Tree:PExpTreeNode;Derive:boolean;GColor:TColor);
var IX,IY:integer; Y:double; HaveLast:boolean;
begin with Canvas do begin
  Application.ProcessMessages;
  HaveLast:=false; IY:=CY;
  Pen.Color:=GColor;
  for IX:=0 to (SX-1) do begin
    Variables['x']:=InvX(IX);
    Y:=Evaluate(Tree);
    if Derive AND not(MathError) then begin
      Variables['x']:=InvX(IX)+DiffStep;
      Y:=(Evaluate(Tree)-Y)/DiffStep;
    end;
    if MathError OR (Y<RYmin) OR (Y>RYmax) then begin
      if HaveLast then Pixels[IX-1,IY]:=GColor;
      HaveLast:=false;
    end else begin
      IY:=GetY(Y);
      if HaveLast then LineTo(IX,IY) else MoveTo(IX,IY);
      HaveLast:=true;
    end;
  end;
end; end;

procedure TGraphForm.DrawParametric(TreeX,TreeY:PExpTreeNode;Derive:boolean;GColor:TColor);
var LX,LY,IterCount:integer; X,Y,T:double; HaveLast:boolean;
begin with Canvas do begin
  Application.ProcessMessages;
  HaveLast:=false;
  Y:=0.0; LX:=CX; LY:=CY;
  Pen.Color:=GColor;
  T:=ParamStart; IterCount:=0;
  while T<=ParamEnd+ParamDiffStep do begin
    Variables['t']:=T;
    X:=Evaluate(TreeX);
    if Derive AND not(MathError) then begin
      Variables['t']:=T+ParamDiffStep;
      X:=(Evaluate(TreeX)-X)/ParamDiffStep;
    end;
    if not(MathError) AND ((X<RXmin) OR (X>RXmax)) then MathError:=true;
    if not MathError then begin
      Variables['x']:=X;
      Y:=Evaluate(TreeY);
      if Derive AND not(MathError) then begin
        Variables['t']:=T;
        Y:=(Y-Evaluate(TreeY))/ParamDiffStep;
      end;
      if not(MathError) AND ((Y<RYmin) OR (Y>RYmax)) then MathError:=true;
    end;
    if MathError then begin
      if HaveLast then Pixels[LX,LY]:=GColor;
      HaveLast:=false;
    end else begin
      LX:=GetX(X); LY:=GetY(Y);
      if HaveLast then LineTo(LX,LY) else MoveTo(LX,LY);
      HaveLast:=true;
    end;
    inc(IterCount); T:=ParamStart+IterCount*ParamStep;
  end;
end; end;

//////////////////////////////////////////////////////////////////// <FormPaint>
procedure TGraphForm.FormPaint(Sender: TObject);
var IX,IY,G,P,j,Pmax:integer;

  procedure DoDrawScale;
  var G,dStart,dEnd,Freq,dText,TH:integer; s:string;
  begin with Canvas do begin
    Pen.Color:=AxeColor;
    // Achsen zeichnen
    IX:=GetX(0.0); if(IX>=0) AND (IX<SX) then begin MoveTo(IX,0); LineTo(IX,SY); end;
    IY:=GetY(0.0); if(IY>=0) AND (IY<SY) then begin MoveTo(0,IY); LineTo(SX,IY); end;
    if not DrawScale then exit;
    Brush.Style:=bsClear;
    // X-Skale: Schriftbreite bestimmen
    P:=0;
    for G:=GXmin to GXmax do begin
      j:=TextWidth(FloatToStr(G*StepX));
      if j>P then P:=j;
    end;
    Freq:=Ceil(P/(ScaleX*StepX));
    // X-Skale: Layout bestimmen
    TH:=TextHeight('-0.123456789');
    dStart:=GetY(0.0)-TICKSIZE; dEnd:=dStart+2*TICKSIZE; dText:=dEnd+TEXTSPACINGY;
    if dEnd>=SY then begin
      dStart:=SY-TICKSIZE; dEnd:=SY; dText:=dStart-TEXTSPACINGY-TH;
    end else if dStart<0 then begin
      dStart:=0; dEnd:=TICKSIZE; dText:=dEnd+TEXTSPACINGY;
    end else if dText+TEXTSPACINGY+TH>SY then
      dText:=dStart-TEXTSPACINGY-TH;
    // X-Skale zeichnen
    for G:=GXmin to GXmax do if (abs(G) MOD Freq)=0 then begin
      IX:=GetX(G*StepX);
      MoveTo(IX,dStart);
      LineTo(IX,dEnd);
      s:=FloatToStr(G*StepX);
      TextOut(IX-(TextWidth(s)-1) DIV 2,dText,s);
    end;
    // Y-Skale: Schriftbreite bestimmen
    P:=0;
    for G:=GYmin to GYmax do begin
      j:=TextWidth(FloatToStr(G*StepY));
      if j>P then P:=j;
    end;
    // Y-Skale: Layout bestimmen
    TH:=(TH+1) DIV 2;
    dStart:=GetX(0.0)-TICKSIZE; dEnd:=dStart+2*TICKSIZE; dText:=-1;
    if dEnd>=SX then begin
      dStart:=SX-TICKSIZE; dEnd:=SX; dText:=-1;
    end else if dStart<0 then begin
      dStart:=0; dEnd:=TICKSIZE; dText:=1;
    end else if dStart-TEXTSPACINGX-P<0 then
      dText:=1;
    // Y-Skale zeichnen
    for G:=GYmin to GYmax do begin
      IY:=GetY(G*StepY);
      MoveTo(dStart,IY);
      LineTo(dEnd,IY);
      s:=FloatToStr(G*StepY);
      if dText<0 then TextOut(dStart-TEXTSPACINGX-TextWidth(s),IY-TH,s)
                 else TextOut(dEnd+TEXTSPACINGX,IY-TH,s);
    end;
  end; end;

  procedure AutoDrawFunction(const fn:TFunction;GColor:TColor);
  begin
    with fn do
      if not Parametric then DrawFunction(Tree,Derive,GColor)
                        else DrawParametric(Tree,Tree2,Derive,GColor);
  end;

begin with Canvas do begin
  if DrawGrid then begin
    // Gitter
    Pen.Color:=GridColor;
    for G:=GXmin to GXmax do if (G<>0) OR not(DrawAxes) then
      begin IX:=GetX(G*StepX); MoveTo(IX,0); LineTo(IX,SY); end;
    for G:=GYmin to GYmax do if (G<>0) OR not(DrawAxes) then
      begin IY:=GetY(G*StepY); MoveTo(0,IY); LineTo(SX,IY); end;
  end;
  // Achsen und Skalen zeichnen -- wird von DoDrawScale alles erledigt
  if DrawAxes AND (not(ForegroundScale) OR MouseMove) then DoDrawScale;
  // Graphen zeichnen
  if MouseMove then exit;
  for G:=0 to High(Functions) do
    with Functions[G] do if Used AND Active AND not(Error) then begin
      if Multi then begin
        try
          Pmax:=abs(Floor((MultiStart-MultiEnd)/MultiStep));
        except
          Pmax:=0;
        end;
        for P:=0 to Pmax do begin
          Variables['a']:=MultiStart+P*MultiStep;
          if Gradient then AutoDrawFunction(Functions[G],RGB(
                                            (cR(Color)*(Pmax-P)+cR(Color2)*P) DIV Pmax,
                                            (cG(Color)*(Pmax-P)+cG(Color2)*P) DIV Pmax,
                                            (cB(Color)*(Pmax-P)+cB(Color2)*P) DIV Pmax))
          else AutoDrawFunction(Functions[G],Color);
        end;
      end else
        AutoDrawFunction(Functions[G],Color);
    end;
  if DrawAxes AND ForegroundScale then DoDrawScale;
end; end;
/////////////////////////////////////////////////////////////////// </FormPaint>

procedure TGraphForm.FormResize(Sender: TObject);
begin
  SX:=ClientWidth;
  SY:=ClientHeight;
  CX:=SX DIV 2;
  CY:=SY DIV 2;
  RangeChange;
end;

procedure TGraphForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ControlForm.Close;
end;

procedure TGraphForm.RangeChange;
begin
  MinX:=InvX(0); MaxX:=InvX(SX-1);
  MaxY:=InvY(0); MinY:=InvY(SY-1);
  RXmin:=InvX(-SX); RXmax:=InvX(SX SHL 1);
  RYmin:=InvY(SY SHL 1); RYmax:=InvY(-SY);
  if AutoGrid then begin
    StepX:=exp(-floor(log10(ScaleX/AUTOGRIDSPEC))*LN10);
    StepY:=exp(-floor(log10(ScaleY/AUTOGRIDSPEC))*LN10);
  end;
  GXmin:=Floor(MinX/StepX); GXmax:=Ceil(MaxX/StepX);
  GYmin:=Floor(MinY/StepY); GYmax:=Ceil(MaxY/StepY);
  DiffStep:=DIFFSTEPFACTOR/ScaleX;
  Invalidate;
  with ControlForm do begin
    EXRangeStart.Text:=FloatToStr(MinX);
    EXRangeEnd.Text:=FloatToStr(MaxX);
    EXRangeStep.Text:=FloatToStr(StepX);
    EYRangeStart.Text:=FloatToStr(MinY);
    EYRangeEnd.Text:=FloatToStr(MaxY);
    EYRangeStep.Text:=FloatToStr(StepY);
  end;
end;

procedure TGraphForm.SetViewRange(Xmin,Xmax,Xstep,Ymin,Ymax,Ystep:double);
var Raus:boolean;
begin
  Raus:=false;
  if (Xstep<0) then begin
    MessageDlg('Das Feld Gitterweite X'+^M'enthlt einen negativen Wert!',mtError,[mbOK],0);
  Raus:=true; end;
  if (Xstep<0) then begin
    MessageDlg('Das Feld Gitterweite Y'+^M'enthlt einen negativen Wert!',mtError,[mbOK],0);
  Raus:=true; end;
  if (Xmax<=Xmin) then begin
    MessageDlg('Der X-Bereich ist ungltig!',mtError,[mbOK],0);
  Raus:=true; end;
  if (Ymax<=Ymin) then begin
    MessageDlg('Der Y-Bereich ist ungltig!',mtError,[mbOK],0);
  Raus:=true; end;
  if Raus then exit;
  CenterX:=(Xmin+Xmax)*0.5;
  CenterY:=(Ymin+Ymax)*0.5;
  ScaleX:=(SX-1)/(Xmax-Xmin);
  ScaleY:=(SY-1)/(Ymax-Ymin);
  StepX:=Xstep; StepY:=Ystep;
  RangeChange;
end;

procedure TGraphForm.SetTRange(Tmin,Tmax,Tstep:double);
var Raus:boolean;
begin
  Raus:=false;
  if (Tstep<0) then begin
    MessageDlg('Das Feld Schrittweite T'+^M'enthlt einen negativen Wert!',mtError,[mbOK],0);
  Raus:=true; end;
  if (Tmax<=Tmin) then begin
    MessageDlg('Der T-Bereich ist ungltig!',mtError,[mbOK],0);
  Raus:=true; end;
  if Raus then exit;
  ParamStart:=Tmin;
  ParamEnd:=Tmax;
  ParamStep:=Tstep;
  ParamDiffStep:=PARAMDIFFSTEPFACTOR*ParamStep;
  Invalidate;
end;

procedure TGraphForm.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var MX,MY:integer; RX,RY:double;
begin
  MousePos:=ScreenToClient(MousePos);
  MX:=MousePos.X-CX; MY:=CY-MousePos.Y;
  RX:=CenterX+MX/ScaleX; RY:=CenterY+MY/ScaleY;
  if WheelDelta>0 then begin
    ScaleX:=ScaleX*SCALESTEP;
    ScaleY:=ScaleY*SCALESTEP;
  end else begin
    ScaleX:=ScaleX/SCALESTEP;
    ScaleY:=ScaleY/SCALESTEP;
  end;
  CenterX:=RX-MX/ScaleX;
  CenterY:=RY-MY/ScaleY;
  RangeChange;
end;

procedure TGraphForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button=mbRight then ControlForm.SetFocus;
  if Button<>mbLeft then exit;
  MouseMove:=true;
  MDX:=X; MDY:=Y;
  MBX:=CenterX; MBY:=CenterY;
end;

procedure TGraphForm.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button<>mbLeft then exit;
  if MouseMove then begin
    MouseMove:=false;
    RangeChange;
  end;
end;

procedure TGraphForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if MouseMove then begin
    CenterX:=MBX+(MDX-X)/ScaleX;
    CenterY:=MBY-(MDY-Y)/ScaleY;
    RangeChange;
  end else
    Caption:='X='+FloatToStr(InvX(X))+' Y='+FloatToStr(InvY(Y));
end;

procedure TGraphForm.FormKeyPress(Sender: TObject; var Key: Char);
var cp:TPoint; wd:integer; Dummy:boolean;
begin
  if (Key='+') OR (Key='-') then begin
    GetCursorPos(cp);
    cp:=ScreenToClient(cp);
    if (cp.X<0) OR (cp.Y<0) OR (cp.X>=SX) OR (cp.Y>=SY) then
      begin cp.X:=CX; cp.Y:=CY; end;
    if Key='+' then wd:=120 else wd:=-120;
    FormMouseWheel(nil,[],wd,ClientToScreen(cp),Dummy);
  end;
end;

end.
