unit Manage2D;
interface
uses SysUtils, Controls, StdCtrls, Dialogs, fparse;

type TFunction=record
                 Used:boolean;
                 Active:boolean;
                 Expression:shortstring;
                 Tree:PExpTreeNode;
                 Parametric:boolean;
                 Expression2:shortstring;
                 Tree2:PExpTreeNode;
                 Error:boolean;
                 Multi:boolean;
                 MultiStart,MultiEnd,MultiStep:double;
                 Color:cardinal;
                 Gradient:boolean;
                 Color2:cardinal;
                 Derive:boolean;
               end;

var Functions:array of TFunction;
    ActiveFunction:integer;

var CenterX,CenterY:double;
    ScaleX,ScaleY:double;
    StepX,StepY:double;
    AutoGrid:boolean;
    ParamStart,ParamEnd,ParamStep:double;

var DrawAxes,DrawGrid,DrawScale:boolean;
    AxeColor,GridColor:cardinal;
    ForegroundScale:boolean;

function CreateNewFunction:integer;
function ParseFunction(const Expression:string;var Tree:PExpTreeNode):boolean;
procedure FormToFunction(Index:integer);
procedure FunctionToForm(Index:integer);

procedure ResetViewRange;

procedure EditToFloat(Edit:TWinControl;var f:double);

function EncodeColor(i:integer):string;
function DecodeColor(const s:string;Default:cardinal):cardinal;

function cR(i:cardinal):integer;
function cG(i:cardinal):integer;
function cB(i:cardinal):integer;
function RGB(r,g,b:integer):cardinal;

implementation
uses Control2D;

function CreateNewFunction:integer;
var i:integer;
begin
  Result:=-1;
  for i:=0 to High(Functions) do
    if not Functions[i].Used then begin
      Result:=i;
      break;
    end;
  if Result<0 then begin
    Result:=length(Functions);
    SetLength(Functions,Result+1);
  end;
  with Functions[Result] do begin
    Used:=true; Active:=true;
    Parametric:=false; Expression:=''; Tree:=nil;
    Expression2:=''; Tree2:=nil; Error:=true;
    Multi:=false; MultiStart:=0.0; MultiEnd:=0.0; MultiStep:=1.0;
    Color:=0; Gradient:=false; Color2:=0;
    Derive:=false;
  end;
end;

function ParseFunction(const Expression:string;var Tree:PExpTreeNode):boolean;
begin
  DestroyExpressionTree(Tree);
  Tree:=ParseExpression(Expression);
  Result:=ParseError;
  if Result then DestroyExpressionTree(Tree);
end;

procedure FormToFunction(Index:integer);
begin
  with ControlForm,Functions[Index] do begin
    if EFunction.Visible then begin
      Parametric:=false;
      Expression:=EFunction.Text;
      Error:=ParseFunction(Expression,Tree);
      DestroyExpressionTree(Tree2);
    end else begin
      Parametric:=true;
      Expression:=EParamX.Text;
      Expression2:=EParamY.Text;
      Error:=ParseFunction(Expression,Tree);
      if Error then DestroyExpressionTree(Tree2)
               else Error:=ParseFunction(Expression2,Tree2);
    end;
    Multi:=CBMulti.Checked;
    EditToFloat(EMultiStart,MultiStart);
    EditToFloat(EMultiEnd,MultiEnd);
    EditToFloat(EMultiStep,MultiStep);
    Color:=PColor1.Color;
    Gradient:=Multi AND CBColor2.Checked;
    Color2:=PColor2.Color;
    Derive:=CBDerive.Checked;
  end;
end;

procedure FunctionToForm(Index:integer);
begin
  with ControlForm,Functions[Index] do begin
    if not Parametric then begin
      EFunction.Visible:=true;
      PParametric.Visible:=false;
      PParamSwitch.Caption:='f(x)=';
      EFunction.Text:=Expression;
      EParamX.Text:='t';
      EParamY.Text:=Expression;
    end else begin
      EFunction.Visible:=false;
      PParametric.Visible:=true;
      PParamSwitch.Caption:='x(t)=';
      EFunction.Text:='';
      EParamX.Text:=Expression;
      EParamY.Text:=Expression2;
    end;
    CBMulti.Checked:=Multi;
    EMultiStart.Text:=FloatToStr(MultiStart);
    EMultiEnd.Text:=FloatToStr(MultiEnd);
    EMultiStep.Text:=FloatToStr(MultiStep);
    PColor1.Color:=Color;
    CBColor2.Checked:=Gradient;
    PColor2.Color:=Color2;
    CBDerive.Checked:=Derive;
    CBMultiClick(nil);
    CBColor2Click(nil);
  end;
end;

procedure ResetViewRange;
begin
  CenterX:=0.0; CenterY:=0.0;
  ScaleX:=32.0; ScaleY:=32.0;
  StepX:=1.0; StepY:=1.0;
end;

procedure EditToFloat(Edit:TWinControl;var f:double);
var s:string; Tree:PExpTreeNode;
begin
  s:='';
  if Edit is TEdit then s:=(Edit as TEdit).Text;
  if Edit is TComboBox then s:=(Edit as TComboBox).Text;
  if s='' then exit;
  Tree:=ParseExpression(s);
  if not ParseError then
    f:=Evaluate(Tree);
  DestroyExpressionTree(Tree);
  if ParseError OR MathError then
    MessageDlg('Das Feld '+Edit.Hint+^M'enthlt keinen gltigen Wert!',mtError,[mbOK],0);
end;

function EncodeColor(i:integer):string;
const hex:array[0..15]of char='0123456789ABCDEF';
begin
  Result:='#'+hex[(i SHR 4)AND 15]+hex[i AND 15]+
    hex[(i SHR 12)AND 15]+hex[(i SHR 8)AND 15]+
    hex[(i SHR 20)AND 15]+hex[(i SHR 16)AND 15];
end;

function DecodeColor(const s:string;Default:cardinal):cardinal;
var i,v:integer;
begin
  Result:=Default;
  if (length(s)<>7) OR (s[1]<>'#') then exit;
  v:=0;
  for i:=2 to 7 do
    case s[i] of
      '0'..'9':v:=(v SHL 4) OR (byte(s[i]) AND 15);
      'A'..'F':v:=(v SHL 4) OR (byte(s[i])-55);
      'a'..'f':v:=(v SHL 4) OR (byte(s[i])-87);
    else
      exit;
    end;
  Result:=((v AND $FF) SHL 16) OR (v AND $FF00) OR (v SHR 16);
end;

function cR(i:cardinal):integer; begin cR:= i AND $0000FF; end;
function cG(i:cardinal):integer; begin cG:=(i AND $00FF00) SHR 8; end;
function cB(i:cardinal):integer; begin cB:=(i AND $FF0000) SHR 16; end;
function RGB(r,g,b:integer):cardinal; begin RGB:=(b SHL 16) OR (g SHL 8) OR r; end;

initialization
  SetLength(Functions,0);
  ActiveFunction:=-1;
  ResetViewRange;
finalization
  Finalize(Functions);
end.
