unit PSUI2D;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons;

const in2pt=72.0;        pt2in=1.0/in2pt;
      in2cm=2.54;        cm2in=1.0/in2cm;
      cm2pt=in2pt/in2cm; pt2cm=1.0/cm2pt;

type TFloatPoint=record X,Y:double; end;

type
  TPSUIForm = class(TForm)
    GBPageSize: TGroupBox;
    Unit_pt: TRadioButton;
    Unit_in: TRadioButton;
    Unit_cm: TRadioButton;
    LUnit: TLabel;
    PaperSizes: TComboBox;
    Bevel1: TBevel;
    BXMargin: TBevel;
    EMargin: TEdit;
    LMargin: TLabel;
    LMarginUnit: TLabel;
    BWinToPage: TBitBtn;
    BPageToWin: TBitBtn;
    GBLineWidths: TGroupBox;
    PFunctionLineWith: TPanel;
    EFunctionLineWidth: TComboBox;
    Lpt2: TLabel;
    LFunctionLineWidth: TLabel;
    PClipBoxLineWidth: TPanel;
    Lpt3: TLabel;
    LClipBoxLineWidth: TLabel;
    EClipBoxLineWidth: TComboBox;
    PAxeLineWidth: TPanel;
    Lpt4: TLabel;
    LAxeLineWidth: TLabel;
    EAxeLineWidth: TComboBox;
    PGridLineWidth: TPanel;
    Lpt5: TLabel;
    LGridLineWidth: TLabel;
    EGridLineWidth: TComboBox;
    PTickLineWidth: TPanel;
    Lpt6: TLabel;
    LTickLineWidth: TLabel;
    ETickLineWidth: TComboBox;
    PNumberFontSize: TPanel;
    Lpt1: TLabel;
    LNumberFontSize: TLabel;
    ENumberFontSize: TComboBox;
    CBDrawClipBox: TCheckBox;
    LWidths: TLabel;
    BXWidths: TBevel;
    PCustomFmt: TPanel;
    LCustomFmtX: TLabel;
    LCustomFmtUnit: TLabel;
    ECustomFmtX: TEdit;
    ECustomFmtY: TEdit;
    BExport: TBitBtn;
    BClose: TBitBtn;
    DSavePS: TSaveDialog;
    procedure PaperSizesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure UnitChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaperSizesChange(Sender: TObject);
    procedure BPageToWinClick(Sender: TObject);
    procedure BWinToPageClick(Sender: TObject);
    procedure BCloseClick(Sender: TObject);
    procedure BExportClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    pt2Unit,Unit2pt:double;
    function GetCurrentSize:TPoint;
    function GetMargin:integer;
    function GetNativeSize(Index:integer):TPoint;
    procedure SetNativeSize(const ptsz:TPoint);
    function UnitSize(const ptSize:TPoint):TFloatPoint;
    function PointSize(const uSize:TFloatPoint):TPoint;
  public
    { Public declarations }
    procedure ExportValues;
    procedure ImportValues;
  end;

var
  PSUIForm: TPSUIForm;

function FloatPoint(X,Y:double):TFloatPoint;
function r10(f:double):string;

implementation
uses Manage2D, GraphW2D, PostScript2D;

{$R *.dfm}

function FloatPoint(X,Y:double):TFloatPoint; begin Result.X:=X; Result.Y:=Y; end;
function r10(f:double):string; begin r10:=FloatToStr(0.1*round(10.0*f)); end;

procedure TPSUIForm.FormCreate(Sender: TObject);
begin
  UnitChange(nil);
end;

procedure TPSUIForm.PaperSizesDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var sFormatName:string; p:integer; ptsz:TPoint; usz:TFloatPoint;
begin
  with PaperSizes.Canvas do begin
    FillRect(Rect);
    sFormatName:=PaperSizes.Items[Index];
    p:=pos('|',sFormatName);
    if p>0 then SetLength(sFormatName,p-1);
    ptsz:=GetNativeSize(Index);
    if (ptsz.X>0) AND (ptsz.Y>0) then begin
      usz:=UnitSize(ptsz);
      TextOut(Rect.Left+2,Rect.Top+1,sFormatName+' ('+
              r10(usz.X)+'x'+r10(usz.Y)+#32+UnitName+')');
    end else
      TextOut(Rect.Left+2,Rect.Top+1,sFormatName);
  end;
end;

procedure TPSUIForm.UnitChange(Sender: TObject);
var OldSize:TPoint; OldMargin:integer; NewSize:TFloatPoint;
begin
  OldSize:=GetCurrentSize;
  OldMargin:=GetMargin;
  if Unit_pt.Checked then begin
    pt2Unit:=1.0; Unit2pt:=1.0; UnitName:='pt';
  end else if Unit_in.Checked then begin
    pt2Unit:=pt2in; Unit2pt:=in2pt; UnitName:='in';
  end else begin
    pt2Unit:=pt2cm; Unit2pt:=cm2pt; UnitName:='cm';
  end;
  LCustomFmtUnit.Caption:=UnitName;
  LMarginUnit.Caption:=UnitName;
  if Assigned(Sender) then begin
    EMargin.Text:=r10(OldMargin*pt2Unit);
    if PaperSizes.ItemIndex=0 then begin
      NewSize:=UnitSize(OldSize);
      ECustomFmtX.Text:=r10(NewSize.X);
      ECustomFmtY.Text:=r10(NewSize.Y);
    end;
  end;
  PaperSizes.Invalidate;
  PaperSizesChange(nil);
end;

procedure TPSUIForm.PaperSizesChange(Sender: TObject);
var Custom:boolean; usz:TFloatPoint;
begin
  Custom:=(PaperSizes.ItemIndex=0);
  ECustomFmtX.Enabled:=Custom;
  ECustomFmtY.Enabled:=Custom;
  if not Custom then begin
    usz:=UnitSize(GetNativeSize(PaperSizes.ItemIndex));
    ECustomFmtX.Text:=r10(usz.X);
    ECustomFmtY.Text:=r10(usz.Y);
  end;
end;

function TPSUIForm.GetCurrentSize:TPoint;
var usz:TFloatPoint;
begin
  if PaperSizes.ItemIndex=0 then begin
    usz.X:=0.0; usz.Y:=0.0;
    EditToFloat(ECustomFmtX,usz.X);
    EditToFloat(ECustomFmtY,usz.Y);
    Result:=PointSize(usz);
  end else
    Result:=GetNativeSize(PaperSizes.ItemIndex);
end;

function TPSUIForm.GetMargin:integer;
var f:double;
begin
  f:=0.0;
  EditToFloat(EMargin,f);
  Result:=round(f*Unit2pt);
end;

function TPSUIForm.GetNativeSize(Index:integer):TPoint;
var s:string; p:integer;
begin
  s:=PaperSizes.Items[Index];
  p:=pos('|',s);
  if p<=0 then begin
    Result.X:=0; Result.Y:=0;
  exit; end;
  s:=copy(s,p+1,length(s)-p);
  p:=pos('x',s);
  if p<=0 then begin
    Result.X:=0; Result.Y:=0;
  exit; end;
  try
    Result.X:=StrToInt(copy(s,1,p-1));
    Result.Y:=StrToInt(copy(s,p+1,length(s)-p));
  except
    Result.X:=0;
    Result.Y:=0;
  end;
end;

procedure TPSUIForm.SetNativeSize(const ptsz:TPoint);
var i:integer; cmpsz:TPoint; usz:TFloatPoint;
begin
  for i:=1 to (PaperSizes.Items.Count-1) do begin
    cmpsz:=GetNativeSize(i);
    if (ptsz.X=cmpsz.X) AND (ptsz.Y=cmpsz.Y) then begin
      PaperSizes.ItemIndex:=i;
      PaperSizesChange(nil);
      exit;
    end;
  end;
  usz:=UnitSize(ptsz);
  ECustomFmtX.Enabled:=true;
  ECustomFmtY.Enabled:=true;
  ECustomFmtX.Text:=r10(usz.X);
  ECustomFmtY.Text:=r10(usz.Y);
  PaperSizes.ItemIndex:=0;
end;

function TPSUIForm.UnitSize(const ptSize:TPoint):TFloatPoint;
begin
  Result.X:=ptSize.X*pt2Unit;
  Result.Y:=ptSize.Y*pt2Unit;
end;

function TPSUIForm.PointSize(const uSize:TFloatPoint):TPoint;
begin
  Result.X:=round(uSize.X*Unit2pt);
  Result.Y:=round(uSize.Y*Unit2pt);
end;

procedure TPSUIForm.BPageToWinClick(Sender: TObject);
var ptsz:TPoint; Margin:integer;
begin
  ptsz:=GetCurrentSize;
  Margin:=2*GetMargin;
  dec(ptsz.X,Margin); dec(ptsz.Y,Margin);
  if (ptsz.X<=0) OR (ptsz.Y<=0) then begin
    Beep; exit;
  end;
  GraphForm.ClientWidth:=ptsz.X;
  GraphForm.ClientHeight:=ptsz.Y;
  GraphForm.Invalidate;
end;

procedure TPSUIForm.BWinToPageClick(Sender: TObject);
var Margin:integer; 
begin
  Margin:=2*GetMargin;
  SetNativeSize(Point(GraphForm.ClientWidth+Margin,GraphForm.ClientHeight+Margin));
end;

procedure TPSUIForm.BCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TPSUIForm.BExportClick(Sender: TObject);
begin
  if not DSavePS.Execute then exit;
  ExportValues;
  WritePSFile(DSavePS.FileName);
end;

procedure TPSUIForm.ExportValues;
var ptsz:TPoint;
begin
  ptsz:=GetCurrentSize;
  if (ptsz.X>0) AND (ptsz.Y>0) then begin
    PageSizeX:=ptsz.X;
    PageSizeY:=ptsz.Y;
  end;
  PageMargin:=GetMargin;
  DrawClipBox:=CBDrawClipBox.Checked;
  EditToFloat(ENumberFontSize,NumberFontSize);
  EditToFloat(EFunctionLineWidth,FunctionLineWidth);
  EditToFloat(EClipBoxLineWidth,ClipBoxLineWidth);
  EditToFloat(EAxeLineWidth,AxeLineWidth);
  EditToFloat(EGridLineWidth,GridLineWidth);
  EditToFloat(ETickLineWidth,TickLineWidth);
end;

procedure TPSUIForm.ImportValues;
begin
  if UnitName='cm' then Unit_cm.Checked:=true
  else if UnitName='in' then Unit_in.Checked:=true
  else Unit_pt.Checked:=true;
  UnitChange(nil);
  SetNativeSize(Point(PageSizeX,PageSizeY));
  EMargin.Text:=r10(PageMargin*pt2Unit);
  CBDrawClipBox.Checked:=DrawClipBox;
  ENumberFontSize.Text:=FloatToStr(NumberFontSize);
  EFunctionLineWidth.Text:=FloatToStr(FunctionLineWidth);
  EClipBoxLineWidth.Text:=FloatToStr(ClipBoxLineWidth);
  EAxeLineWidth.Text:=FloatToStr(AxeLineWidth);
  EGridLineWidth.Text:=FloatToStr(GridLineWidth);
  ETickLineWidth.Text:=FloatToStr(TickLineWidth);
end;

procedure TPSUIForm.Button1Click(Sender: TObject);
begin
  ExportValues;
  ImportValues;
end;

end.
