unit PostScript2D;
interface
uses SysUtils, Math, fparse;

var DrawClipBox:boolean;  // Clipping-Box zeichnen?
    FunctionLineWidth,ClipBoxLineWidth,AxeLineWidth,GridLineWidth,
      TickLineWidth:double;  // verschiedene Linienstrken
    NumberFontSize:double;  // Gre und Hhe der Schriftart
    PageSizeX,PageSizeY,PageMargin:integer;  // Seitenmae
    UnitName:string[3];  // Maeinheit (frs Kontrollformular)

function WritePSFile(const FileName:string):boolean;

implementation
uses GraphW2D, Manage2D;

const FONTHEIGHTFACTOR=0.725;  // Schtzung der Schrifthhe
      PSTEXTSPACING=2;  // Abstand Text<->Tick (in PS einheitlicher als auf'm Screen)
      // Schtzungen der Breiten einiger Zeichen:
      PSF_DOTWIDTH=0.278;
      PSF_MINUSWIDTH=0.333;
      PSF_NUMBERWIDTH=0.556;
      PSF_EXPWIDTH=0.667;

var SX,SY:integer;  // Ausgabegre in pt
    MinX,MinY,MaxX,MaxY:double;  // sichtbarer Ausschnitt
    RXmin,RXmax,RYmin,RYmax:double;  // maximale Funktionswerte (alles auerhalb = "unstetig")
    GXmin,GXmax,GYmin,GYmax:integer;  // sichtbarer Gitterausschnitt
    DiffStep,ParamDiffStep:double;  // Schrittweite zur numerischen Differentiation
    Encapsulated:boolean;  // EPS oder Plain PostScript?
    PageCenterX,PageCenterY:integer;  // Seitenmitte

var PS:text;

function GetX(X:double):double; begin GetX:=(X-CenterX)*ScaleX; end;
function GetY(Y:double):double; begin GetY:=(Y-CenterY)*ScaleY; end;
function InvX(IX:integer):double; begin InvX:=IX/ScaleX+CenterX; end;
function InvY(IY:integer):double; begin InvY:=IY/ScaleY+CenterY; end;

function PSColor(r,g,b:integer):string; overload;
  begin Result:=
  FloatToStr(r/255.0)+#32+FloatToStr(g/255.0)+#32+FloatToStr(b/255.0)+
  ' setrgbcolor';end;
function PSColor(col:cardinal):string; overload;
  begin Result:=PSColor(cR(col),cG(col),cB(col)); end;

function PSTextWidth(const s:string):double;
var i:integer;
begin
  Result:=0.0;
  for i:=1 to length(s) do
    case s[i] of
      '-':Result:=Result+PSF_MINUSWIDTH;
      '.':Result:=Result+PSF_DOTWIDTH;
      '0'..'9':Result:=Result+PSF_NUMBERWIDTH;
      'E':Result:=Result+PSF_EXPWIDTH;
    end;
  Result:=Result*NumberFontSize;
end;

  procedure DoDrawScale;
  var G,Freq:integer; P,j,dStart,dEnd,dXText,dYTextMode,TH,FP:double;
  begin
    // Achsen zeichnen
    writeln(PS);
    writeln(PS,'% Axes');
    writeln(PS,'newpath');
    FP:=GetX(0.0); if (FP>=-SX) AND (FP<=SX) then
      writeln(PS,FloatToStr(FP),#32,-SY,' moveto 0 ',2*SY,' rlineto');
    FP:=GetY(0.0); if (FP>=-SY) AND (FP<=SY) then
      writeln(PS,-SX,#32,FloatToStr(FP),' moveto ',2*SX,' 0 rlineto');
    writeln(PS,PSColor(AxeColor),#32,FloatToStr(AxeLineWidth),' setlinewidth stroke');
    // ggf. Skalen zeichnen
    if not DrawScale then exit;
    writeln(PS);
    writeln(PS,'% Scale Ticks');
    writeln(PS,'newpath');
    // X-Skale: Schriftbreite bestimmen
    P:=0;
    for G:=GXmin to GXmax do begin
      j:=PSTextWidth(FloatToStr(G*StepX));
      if j>P then P:=j;
    end;
    Freq:=Ceil(P/(ScaleX*StepX));
    // X-Skale: Layout bestimmen
    TH:=NumberFontSize*FONTHEIGHTFACTOR;
    dStart:=GetY(0.0)-TICKSIZE; dEnd:=dStart+2*TICKSIZE; dXText:=dStart-PSTEXTSPACING-TH;
    if dEnd>SY then begin
      dStart:=SY-TICKSIZE; dEnd:=SY; dXText:=dStart-PSTEXTSPACING-TH;
    end else if dStart<-SY then begin
      dStart:=-SY; dEnd:=TICKSIZE-SY; dXText:=dEnd+PSTEXTSPACING;
    end else if dXText+PSTEXTSPACING+NumberFontSize>SY then
      dXText:=dEnd+PSTEXTSPACING;
    // X-Skale zeichnen
    for G:=GXmin to GXmax do if (abs(G) MOD Freq)=0 then begin
      FP:=GetX(G*StepX);
      writeln(PS,FloatToStr(FP),#32,FloatToStr(dStart),' moveto ',
                 FloatToStr(FP),#32,FloatToStr(dEnd),' lineto');
    end;
    // Y-Skale: Schriftbreite bestimmen
    P:=0;
    for G:=GYmin to GYmax do begin
      j:=PSTextWidth(FloatToStr(G*StepY));
      if j>P then P:=j;
    end;
    // Y-Skale: Layout bestimmen
    TH:=0.5*TH;
    dStart:=GetX(0.0)-TICKSIZE; dEnd:=dStart+2*TICKSIZE; dYTextMode:=-1;
    if dEnd>SX then begin
      dStart:=SX-TICKSIZE; dEnd:=SX; dYTextMode:=-1;
    end else if dStart<-SX then begin
      dStart:=-SX; dEnd:=TICKSIZE-SX; dYTextMode:=1;
    end else if dStart-PSTEXTSPACING-P<-SX then
      dYTextMode:=1;
    // Y-Skale zeichnen
    for G:=GYmin to GYmax do begin
      FP:=GetY(G*StepY);
      writeln(PS,FloatToStr(dStart),#32,FloatToStr(FP),' moveto ',
                 FloatToStr(dEnd),#32,FloatToStr(FP),' lineto');
    end;
    writeln(PS,PSColor(AxeColor),#32,FloatToStr(TickLineWidth),' setlinewidth stroke');
    // Beschriftungen
    writeln(PS);
    writeln(PS,'% Scale');
    writeln(PS,'/Helvetica findfont ',FloatToStr(NumberFontSize),
               ' scalefont setfont 0 setgray');
    for G:=GXmin to GXmax do if (abs(G) MOD Freq)=0 then begin
      FP:=GetX(G*StepX);
      writeln(PS,FloatToStr(FP),#32,FloatToStr(dXText),' moveto (',
                 FloatToStr(G*StepX),') center');
    end;
    for G:=GYmin to GYmax do begin
      FP:=GetY(G*StepY)-TH;
      if dYTextMode<0 then write(PS,FloatToStr(dStart-PSTEXTSPACING))
                      else write(PS,FloatToStr(dEnd+PSTEXTSPACING));
      write(PS,#32,FloatToStr(FP),' moveto (',FloatToStr(G*StepY),') ');
      if dYTextMode<0 then writeln(PS,'right') else writeln(PS,'show');
    end;
  end;


procedure DrawFunction(Tree:PExpTreeNode;Derive:boolean;ColR,ColG,ColB:integer);
var IX:integer; X,Y:double; HaveLast:boolean;
begin
  writeln(PS,'newpath');
  HaveLast:=false;
  for IX:=-SX to SX do begin
    X:=InvX(IX);
    Variables['x']:=X;
    Y:=Evaluate(Tree);
    if Derive AND not(MathError) then begin
      Variables['x']:=X+DiffStep;
      Y:=(Evaluate(Tree)-Y)/DiffStep;
    end;
    if MathError OR (Y<RYmin) OR (Y>RYmax) then
      HaveLast:=false
    else begin
      if HaveLast then writeln(PS,IX,#32,FloatToStr(GetY(Y)),' lineto')
                  else writeln(PS,IX,#32,FloatToStr(GetY(Y)),' moveto');
      HaveLast:=true;
    end;
  end;
end;

procedure DrawParametric(TreeX,TreeY:PExpTreeNode;Derive:boolean;ColR,ColG,ColB:integer);
var IterCount:integer; X,Y,T:double; HaveLast:boolean;
begin
  writeln(PS,'newpath');
  HaveLast:=false;
  Y:=0.0;
  T:=ParamStart; IterCount:=0;
  while T<=ParamEnd+ParamDiffStep do begin
    Variables['t']:=T;
    X:=Evaluate(TreeX);
    if Derive AND not(MathError) then begin
      Variables['t']:=T+ParamDiffStep;
      X:=(Evaluate(TreeX)-X)/ParamDiffStep;
    end;
    if not(MathError) AND ((X<RXmin) OR (X>RXmax)) then MathError:=true;
    if not MathError then begin
      Variables['x']:=X;
      Y:=Evaluate(TreeY);
      if Derive AND not(MathError) then begin
        Variables['t']:=T;
        Y:=(Y-Evaluate(TreeY))/ParamDiffStep;
      end;
      if not(MathError) AND ((Y<RYmin) OR (Y>RYmax)) then MathError:=true;
    end;
    if MathError then
      HaveLast:=false
    else begin
      if HaveLast then writeln(PS,FloatToStr(GetX(X)),#32,FloatToStr(GetY(Y)),' lineto')
                  else writeln(PS,FloatToStr(GetX(X)),#32,FloatToStr(GetY(Y)),' moveto');
      HaveLast:=true;
    end;
    inc(IterCount); T:=ParamStart+IterCount*ParamStep;
  end;
end;

procedure AutoDrawFunction(const fn:TFunction;ColR,ColG,ColB:integer);
begin
  writeln(PS);
  with fn do if Parametric then begin
    if Derive then write(PS,'% x(t)=d[',Expression,']/dt; y(t)=d[',Expression2,']/dt')
              else write(PS,'% x(t)=',Expression,'; y(t)=',Expression2);
    if Multi then writeln(PS,'; a=',FloatToStr(Variables['a'])) else writeln(PS);
    DrawParametric(Tree,Tree2,Derive,ColR,ColG,ColB);
  end else begin
    if Derive then write(PS,'% f(x)=d[',Expression,']/dx')
              else write(PS,'% f(x)=',Expression);
    if Multi then writeln(PS,'; a=',FloatToStr(Variables['a'])) else writeln(PS);
    DrawFunction(Tree,Derive,ColR,ColG,ColB);
  end;
  writeln(PS,FloatToStr(FunctionLineWidth),' setlinewidth ',
             PSColor(ColR,ColG,ColB),' stroke');
end;

function WritePSFile(const FileName:string):boolean;
var G,P,Pmax:integer; FP:double;
begin
  // Datei ffnen
  WritePSFile:=false;
  AssignFile(PS,FileName);
  {$I-} Rewrite(PS); {$I+}
  if IOresult<>0 then exit;
  Encapsulated:=(UpperCase(ExtractFileExt(FileName))='.EPS');

  // ViewRange einstellen
  PageCenterX:=PageSizeX SHR 1;
  PageCenterY:=PageSizeY SHR 1;
  SX:=GraphForm.ClientWidth SHR 1;
  SY:=GraphForm.ClientHeight SHR 1;
  MinX:=InvX(-SX); MaxX:=InvX(SX);
  MinY:=InvY(-SY); MaxY:=InvY(SY);
  RXmin:=InvX(-3*SX); RXmax:=InvX(3*SX);
  RYmin:=InvY(-3*SY); RYmax:=InvY(3*SY);
  GXmin:=Floor(MinX/StepX); GXmax:=Ceil(MaxX/StepX);
  GYmin:=Floor(MinY/StepY); GYmax:=Ceil(MaxY/StepY);
  DiffStep:=DIFFSTEPFACTOR/ScaleX;
  ParamDiffStep:=PARAMDIFFSTEPFACTOR*ParamStep;

  // PS-Header schreiben
  if Encapsulated then writeln(PS,'%!PS-Adobe-3.0 EPSF-3.0')
                  else writeln(PS,'%!PS-Adobe-3.0');
  if Encapsulated
    then writeln(PS,'%%BoundingBox: ',
                    PageCenterX-SX,#32,PageCenterY-SY,#32,
                    PageCenterX+SX,#32,PageCenterY+SY,#32)
    else writeln(PS,'%%BoundingBox: 0 0 ',PageSizeX,#32,PageSizeY);
  writeln(PS,'%%Creator: Graph2D');
  writeln(PS,'%%Pages: 1');
  writeln(PS,'%%EndComments');
  writeln(PS,'%%BeginResource: procset Graph2DToolset 1.0 0');
  writeln(PS,'/center{dup stringwidth exch 2 div neg exch 2 div neg rmoveto show}bind def');
  writeln(PS,'/right{dup stringwidth exch neg exch neg rmoveto show}bind def');
  writeln(PS,'%%EndResource');
  writeln(PS,'%%Page: 1 1');
  writeln(PS,PageCenterX,#32,PageCenterY,' translate');

  // Clipping einstellen
  write(PS,'gsave newpath ');
  write(PS,-SX,#32,-SY,' moveto ');
  write(PS,SX,#32,-SY,' lineto ');
  write(PS,SX,#32,SY,' lineto ');
  writeln(PS,-SX,#32,SY,' lineto');
  write(PS,'closepath ');
  if DrawClipBox then writeln(PS,'gsave clip') else writeln(PS,'clip');

  // Gitter zeichnen
  if DrawGrid then begin
    writeln(PS);
    if DrawAxes then writeln(PS,'% Grid and Axes')
                else writeln(PS,'% Grid');
    writeln(PS,'newpath');
    for G:=GXmin to GXmax do if (G<>0) OR not(DrawAxes) then begin
      FP:=GetX(G*StepX);
      writeln(PS,FloatToStr(FP),#32,-SY,' moveto 0 ',2*SY,' rlineto');
    end;
    for G:=GYmin to GYmax do if (G<>0) OR not(DrawAxes) then begin
      FP:=GetY(G*StepY);
      writeln(PS,-SX,#32,FloatToStr(FP),' moveto ',2*SX,' 0 rlineto');
    end;
    writeln(PS,PSColor(GridColor),#32,FloatToStr(GridLineWidth),' setlinewidth stroke');
  end;
  // Achsen und Skalen zeichnen -- wird von DoDrawScale alles erledigt
  if DrawAxes AND not(ForegroundScale) then DoDrawScale;

  // Funktionen zeichnen
  for G:=0 to High(Functions) do
  with Functions[G] do if Used AND Active AND not(Error) then begin
    if Multi then begin
      try
        Pmax:=abs(Floor((MultiStart-MultiEnd)/MultiStep));
      except
        Pmax:=0;
      end;
      for P:=0 to Pmax do begin
        Variables['a']:=MultiStart+P*MultiStep;
        if Gradient then AutoDrawFunction(Functions[G],
                                          (cR(Color)*(Pmax-P)+cR(Color2)*P) DIV Pmax,
                                          (cG(Color)*(Pmax-P)+cG(Color2)*P) DIV Pmax,
                                          (cB(Color)*(Pmax-P)+cB(Color2)*P) DIV Pmax)
        else AutoDrawFunction(Functions[G],cR(Color),cG(Color),cB(Color));
      end;
    end else
      AutoDrawFunction(Functions[G],cR(Color),cG(Color),cB(Color));
  end;
  if DrawAxes AND ForegroundScale then DoDrawScale;

  writeln(PS);
  if DrawClipBox then
    writeln(PS,'grestore ',FloatToStr(ClipBoxLineWidth),' setlinewidth 0 setgray stroke');
  writeln(PS,'grestore showpage');
  writeln(PS,'%%EOF');
  CloseFile(PS);
  WritePSFile:=true;
end;

end.
