unit fparse;
interface
uses SysUtils,Math;

// {$DEFINE VERBOSE_PARSING}

// Werte fr NodeType
const ntConstant=0;
      ntOperator=1;
      ntUnaryFunction=2;
      ntBinaryFunction=3;                  
      ntVariable=4;

// Knoten des Ausdrucksbaums
type PExpTreeNode=^TExpTreeNode;
     TExpTreeNode=record
                    Operator:integer;
                  case NodeType:integer of
                    ntConstant:(Constant:double);
                    ntOperator:(A,B:PExpTreeNode);
                    ntVariable:(Variable:char);
                  end;

// Unre Funktionen
type TUnaryFunctionProc=function(ID:integer;const a:double):double;
type TUnaryFunction=record
                      Name:string[15];
                      ID:integer;
                      Proc:TUnaryFunctionProc;
                    end;
var UnaryFunctions:array of TUnaryFunction;

// Binre Funktionen
type TBinaryFunctionProc=function(ID:integer;const a,b:double):double;
type TBinaryFunction=record
                       Name:string[15];
                       ID:integer;
                       Proc:TBinaryFunctionProc;
                     end;
var BinaryFunctions:array of TBinaryFunction;

// API
const ANS=#96;
var Variables:array[ANS..'z']of double;

var ParseError:boolean;
function ParseExpression(const Expr:string):PExpTreeNode;

var MathError:boolean;
function Evaluate(Tree:PExpTreeNode):double;

type TDumpMode=(dmTree,dmNatural,dmPolish);
function DumpExpressionTree(Tree:PExpTreeNode;DumpMode:TDumpMode):string;

procedure DestroyExpressionTree(var Tree:PExpTreeNode);

implementation

////////////////////////////////////////////////////////////////////////////////
function ParseExpression(const Expr:string):PExpTreeNode;
type TCharSet=set of char;

  function SubSearch(const Expr:string;SearchSet:TCharSet):integer;
  var i,BracketLevel:integer;
  begin
    BracketLevel:=0; Result:=0;
    for i:=length(Expr) downto 1 do
      case Expr[i] of
        '(':begin
              dec(BracketLevel);
              if (BracketLevel<0) then
                begin ParseError:=true; exit; end;
            end;
        ')':inc(BracketLevel);
      else
        if (BracketLevel=0) AND (i>1) AND (Expr[i] IN SearchSet) then
          begin Result:=i; exit; end;
      end;
    if (BracketLevel<>0) then ParseError:=true;
  end;

  function SubExpr(const Expr:string;Level:integer):PExpTreeNode;
  var Len,OpPos,BracketPos,FuncNo:integer;
      ProbableConstant:double;
  begin
    Result:=nil;
    if ParseError then exit;
    Len:=length(Expr);
    if Len<1 then begin ParseError:=true; exit; end;
    {$IFDEF VERBOSE_PARSING} write(^M^J,StringOfChar(#32,Level*2),'+ "',Expr,'": '); {$ENDIF}
    // Zahlen-Konstante ?
    Val(Expr,ProbableConstant,OpPos);
    if OpPos=0 then begin
      New(Result);
      Result^.NodeType:=ntConstant;
      Result^.Constant:=ProbableConstant;
      {$IFDEF VERBOSE_PARSING} write('Constant ',FloatToStr(ProbableConstant)); {$ENDIF}
      exit;
    end;
    // Operatoren suchen
    OpPos:=SubSearch(Expr,['&','|']);
    if OpPos=0 then begin
      OpPos:=SubSearch(Expr,['=','<','>']);
      if OpPos=0 then begin
        OpPos:=SubSearch(Expr,['-','+']);
        if OpPos=0 then begin
          OpPos:=SubSearch(Expr,['*','/']);
          if OpPos=0 then
            OpPos:=SubSearch(Expr,['^']);
        end;
      end;
    end;
    {$IFDEF VERBOSE_PARSING} if ParseError then write('ERROR: Bracket mismatch'); {$ENDIF}
    if ParseError then exit;
    // Operator ?
    if OpPos>0 then begin
      New(Result);
      Result^.NodeType:=ntOperator;
      Result^.Operator:=ord(Expr[OpPos]);
      {$IFDEF VERBOSE_PARSING} write('Operator '#39,Expr[OpPos],#39); {$ENDIF}
      Result^.A:=SubExpr(copy(Expr,1,OpPos-1),Level+1);
      Result^.B:=SubExpr(copy(Expr,OpPos+1,Len-OpPos),Level+1);
      exit;
    end;
    // Klammer/Funktion ?
    if (Expr[Len]=')') then begin
      BracketPos:=pos('(',Expr);
      if BracketPos=1 then begin
        // Klammer:
        {$IFDEF VERBOSE_PARSING} write('Parentheses'); {$ENDIF}
        Result:=SubExpr(copy(Expr,2,Len-2),Level+1);
        exit;
      end else begin
        // Unre Funktion?
        for FuncNo:=0 to High(UnaryFunctions) do
          with UnaryFunctions[FuncNo] do
            if (length(Name)=BracketPos-1) AND (copy(Expr,1,BracketPos-1)=Name) then begin
              New(Result);
              Result^.NodeType:=ntUnaryFunction;
              Result^.Operator:=FuncNo;
              {$IFDEF VERBOSE_PARSING} write('Unary Function "',copy(Expr,1,BracketPos-1),#34); {$ENDIF}
              Result^.A:=SubExpr(copy(Expr,BracketPos+1,Len-BracketPos-1),Level+1);
              exit;
            end;
        // Binre Funktion?
        for FuncNo:=0 to High(BinaryFunctions) do
          with BinaryFunctions[FuncNo] do
            if (length(Name)=BracketPos-1) AND (copy(Expr,1,BracketPos-1)=Name) then begin
              New(Result);
              Result^.NodeType:=ntBinaryFunction;
              Result^.Operator:=FuncNo;
              {$IFDEF VERBOSE_PARSING} write('Binary Function "',copy(Expr,1,BracketPos-1),#34); {$ENDIF}
              OpPos:=pos(',',Expr);
              if OpPos<=BracketPos then begin
                {$IFDEF VERBOSE_PARSING} write(' -- ERROR: missing parameter'); {$ENDIF}
                ParseError:=true;
              end else begin
                Result^.A:=SubExpr(copy(Expr,BracketPos+1,OpPos-BracketPos-1),Level+1);
                Result^.B:=SubExpr(copy(Expr,OpPos+1,Len-OpPos-1),Level+1);
              end;
              exit;
            end;
      end;
    end;
    // Vorzeichen ?
    if Expr[1]='+' then begin
      {$IFDEF VERBOSE_PARSING} write('Sign ''+'''); {$ENDIF}
      Result:=SubExpr(copy(Expr,2,Len-1),Level+1);
      exit;
    end;
    if Expr[1]='-' then begin
      New(Result);
      Result^.NodeType:=ntUnaryFunction;
      Result^.Operator:=0;  // neg() mu IMMER an Stelle Null stehen!!!
      {$IFDEF VERBOSE_PARSING} write('Sign ''-'''); {$ENDIF}
      Result^.A:=SubExpr(copy(Expr,2,Len-1),Level+1);
      exit;
    end;
    // Pi?
    if (Len=2) AND (Expr[1]='p') AND (Expr[2]='i') then begin
      New(Result);
      Result^.NodeType:=ntConstant;
      Result^.Constant:=Pi;
      {$IFDEF VERBOSE_PARSING} write('Pi'); {$ENDIF}
      exit;
    end;
    // Ans?
    if (Len=3) AND (Expr[1]='a') AND (Expr[2]='n') AND (Expr[3]='s') then begin
      New(Result);
      Result^.NodeType:=ntVariable;
      Result^.Variable:=ANS;
      {$IFDEF VERBOSE_PARSING} write('Ans'); {$ENDIF}
      exit;
    end;
    // Variable ?
    if (Len=1) AND (Expr[1] IN ['a'..'z']) then begin
      New(Result);
      Result^.NodeType:=ntVariable;
      Result^.Variable:=Expr[1];
      {$IFDEF VERBOSE_PARSING} write('Variable ',Expr[1]); {$ENDIF}
      exit;
    end;
    // alles nix? dann Fehler!
    {$IFDEF VERBOSE_PARSING} write('ERROR: Invalid Token'); {$ENDIF}
    ParseError:=true;
  end;

begin
  ParseError:=false;
  {$IFDEF VERBOSE_PARSING} write('Parsing Expression "',Expr,'":'); {$ENDIF}
  Result:=SubExpr(LowerCase(Expr),0);
  {$IFDEF VERBOSE_PARSING} writeln; {$ENDIF}
end;

////////////////////////////////////////////////////////////////////////////////
function DumpExpressionTree(Tree:PExpTreeNode;DumpMode:TDumpMode):string;

  procedure DumpAsTree(Node:PExpTreeNode;Level:integer);
  begin
    Result:=Result+^M^J+StringOfChar(#32,Level*2)+'+ ';
    if Node=nil then begin
      Result:=Result+'NULL pointer';
      exit;
    end;
    with Node^ do
      case NodeType of
        ntConstant:Result:=Result+FloatToStr(Constant);
        ntOperator:begin
                     Result:=Result+'a'+char(Operator)+'b';
                     DumpAsTree(A,Level+1);
                     DumpAsTree(B,Level+1);
                   end;
        ntUnaryFunction:begin
                          Result:=Result+UnaryFunctions[Operator].Name+'(a)';
                          DumpAsTree(A,Level+1);
                        end;
        ntBinaryFunction:begin
                           Result:=Result+BinaryFunctions[Operator].Name+'(a,b)';
                           DumpAsTree(A,Level+1);
                           DumpAsTree(B,Level+1);
                         end;
        ntVariable:Result:=Result+'='+Variable;
      end;
  end;

  function Natural(Node:PExpTreeNode):string;
  begin
    if Node=nil then begin
      Result:='NULL';
      exit;
    end;
    with Node^ do
      case NodeType of
        ntConstant:Result:=FloatToStr(Constant);
        ntOperator:Result:='('+Natural(A)+')'+char(Operator)+'('+Natural(B)+')';
        ntUnaryFunction:Result:=UnaryFunctions[Operator].Name+'('+Natural(A)+')';
        ntBinaryFunction:Result:=BinaryFunctions[Operator].Name+'('+Natural(A)+','+Natural(B)+')';
        ntVariable:Result:=Variable;        
      end;
  end;

  function Polish(Node:PExpTreeNode):string;
  begin
    if Node=nil then begin
      Result:='NULL';
      exit;
    end;
    with Node^ do
      case NodeType of
        ntConstant:Result:=FloatToStr(Constant);
        ntOperator:Result:=Polish(A)+#32+Polish(B)+#32+char(Operator);
        ntUnaryFunction:Result:=Polish(A)+#32+UnaryFunctions[Operator].Name;
        ntBinaryFunction:Result:=Polish(A)+#32+Polish(B)+#32+BinaryFunctions[Operator].Name;
        ntVariable:Result:=Variable;        
      end;
  end;

begin
  case DumpMode of
    dmTree:begin
             Result:='';
             DumpAsTree(Tree,0);
             Result:=copy(Result,3,length(Result)-2);
           end;
    dmNatural:Result:=Natural(Tree);
    dmPolish:Result:=Polish(Tree);
  end;
end;

////////////////////////////////////////////////////////////////////////////////
function Evaluate(Tree:PExpTreeNode):double;

  function SubEval(Node:PExpTreeNode):double;
  begin
    Result:=0.0;
    if MathError then exit;
    if Node=nil then begin MathError:=true; exit; end;
    try with Node^ do
      case NodeType of
        ntConstant:Result:=Constant;
        ntOperator:case char(Operator) of
                     '+':Result:=SubEval(A)+SubEval(B);
                     '-':Result:=SubEval(A)-SubEval(B);
                     '*':Result:=SubEval(A)*SubEval(B);
                     '/':Result:=SubEval(A)/SubEval(B);
                     '^':Result:=Power(SubEval(A),SubEval(B));
                     '=':Result:=integer(SubEval(A)=SubEval(B));
                     '<':Result:=integer(SubEval(A)<SubEval(B));
                     '>':Result:=integer(SubEval(A)>SubEval(B));
                     '&':Result:=integer((SubEval(A)<>0.0) AND (SubEval(B)<>0.0));
                     '|':Result:=integer((SubEval(A)<>0.0) OR (SubEval(B)<>0.0));
                   else
                     MathError:=true;
                   end;
        ntUnaryFunction:with UnaryFunctions[Operator] do
          Result:=Proc(ID,SubEval(A));
        ntBinaryFunction:with BinaryFunctions[Operator] do
          Result:=Proc(ID,SubEval(A),SubEval(B));
        ntVariable:Result:=Variables[Variable];
      end;
    except
      Result:=0.0;
      MathError:=true;
    end;
  end;

begin
  MathError:=false;
  Result:=SubEval(Tree);
  Variables[ANS]:=Result;
end;

////////////////////////////////////////////////////////////////////////////////
procedure DestroyExpressionTree(var Tree:PExpTreeNode);
begin
  if Tree=nil then exit;
  with Tree^ do
    case NodeType of
      ntUnaryFunction:DestroyExpressionTree(A);
      ntBinaryFunction,ntOperator:begin
        DestroyExpressionTree(A);
        DestroyExpressionTree(B);
      end;
    end;
  Dispose(Tree);
  Tree:=nil;
end;

////////////////////////////////////////////////////////////////////////////////

{$I fparse_unary.inc.pas}
{$I fparse_binary.inc.pas}

var c:char;
begin
  for c:='a' to 'z' do Variables[c]:=0.0;
  SetLength(UnaryFunctions,NumberOfBasicUnaryFunctions);
  Move(BasicUnaryFunctions[0],UnaryFunctions[0],
       NumberOfBasicUnaryFunctions*sizeof(TUnaryFunction));
  SetLength(BinaryFunctions,NumberOfBasicBinaryFunctions);
  Move(BasicBinaryFunctions[0],BinaryFunctions[0],
       NumberOfBasicBinaryFunctions*sizeof(TBinaryFunction));
end.
